'*******************************************************************************
'* Name    : GIE_SaveRestore.inc                                               *
'* Author  : [Alberto Freixanet]                                               *
'* Purpose : Save/Clear & Restore/Set the GIE                                  *
'* Input   : none                                                              *
'* Output  : none                                                              *
'* Version : 2.0 01/04/2023                                                    *
'* Note    :                                                                   *
'*******************************************************************************
'/ Version 2.1 16/06/2023
'/ Added INTCON0 register.
'/

$ifndef __GIE_SaveRestore
$define __GIE_SaveRestore
'/------------------------------------------------------------------------------
$undef  _WhatLib
$define _WhatLib "GIE_SaveRestore.inc"

$undef _WhatVersion
$define _WhatVersion "Version 2.1 16/06/2023"

'*******************************************************************************
'* Name    : WriteMSG                                                          *
'* Author  : [Alberto Freixanet]                                               *
'* Purpose : Preprocessor Macro To write a comment in the Asm file             *
'*******************************************************************************
$ifndef WriteMSG
$define WriteMSG(pMessage) '
Asm                        '
messg pMessage             '
EndAsm

$endif

'*******************************************************************************

'/ Defined in the DeclareLocalVars.inc.
'/ Global variable.
    $ifndef xINTCON_Save_Var
    $define xINTCON_Save_Var
    Dim xINTCON_Save As Byte Access
    $endif

Proc GIE_Save()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: GIE_Save : START")
    '/-------------------------------------------------------------------------- 
#ifdef __INTERRUPTS_ENABLED
    $ifdef _INTCON
    xINTCON_Save = INTCON 
    INTCONbits_GIE = 0
    $endif
    $ifdef _INTCON0
    xINTCON_Save = INTCON0 
    INTCON0bits_GIE = 0
    $endif
#endif
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: GIE_Save : END")
    '/-------------------------------------------------------------------------- 
EndProc    

Proc GIE_Restore()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: GIE_Restore : START")
    '/--------------------------------------------------------------------------
#ifdef __INTERRUPTS_ENABLED
    $ifdef _INTCON
    If xINTCON_Save.7 = 1 Then
        INTCONbits_GIE = 1
    EndIf        
    $endif
    $ifdef _INTCON0
    If xINTCON_Save.7 = 1 Then
        INTCON0bits_GIE = 1
    EndIf        
    $endif
#endif
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: GIE_Restore : END")
    '/--------------------------------------------------------------------------
EndProc  
'/------------------------------------------------------------------------------
$endif  
'/==============================================================================

