'****************************************************************
'*  Name    : Test_PCF8574A_INT.BAS                             *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2015 [AFreixanet 2015]              *
'*          : All Rights Reserved                               *
'*  Date    : 28/06/2023                                        *
'*  Version : 1.0                                               *
'*  Notes   : Test the PCF8574A expander with Interrupts        *
'*          : 8 bits X 8 expanders = 64 Bits                    *
'*          : Using PROTEUS version 8.2                         *
'* Compiler : Free Positron Version 4.0.1.8                     *
'****************************************************************
'*                                                              *
'* Version 1.0 - 28/06/2023                                     *
'*                                                              *
'****************************************************************
    Include "Amicus18-2.inc"
'/==============================================================================      
    Declare Watchdog = On        
    Declare Create_Coff = On
    Declare Bootloader = Off
'    Declare Icd_Req = OFF
'    Declare Show_System_Variables = On
'*******************************************************************************
'///////////////////////////////////////////////////////////////////////////////
'*******************************************************************************
'/ WARNING: DO NOT MODIFY THE CONFIGURATION REGISTERS => COULD BE A DISASTER!!!
'/ DO NOT COPY SOME CODE FROM THE INCLUDE FILE TO YOUR MAIN PROGRAMM => COULD BE A DISASTER!!!
'/ (ONLY FOR EXPERIMENTED USERS) USE LIBRARY COMMANDS ONLY TO READ/WRITE THE EXPANDER REGISTERS. 
'/ ALL COMMANDS, THAT NOT USED, ARE NOT COMPILED INTO YOUR PIC. (DO NOT WORRY)
'/ THERE ARE A LOT OF CONTROLS, CHECKING THE RETURN ACK, USE THEM TO DEBUG YOUR PROYECT.
'/ IF YOUR PROYECT OVERCOMES THE "PCF8574_Init()" COMMAND THEN => CONGRATULATIONS!!!
'*******************************************************************************
'/////////////////////////////////////////////////////////////////////////////// 
'*******************************************************************************
'/ Uncomment this line to enable debug code
    $define _IntDebug

' Define the Expander type. (See ISIS schematic)
'    $define PCF8574
    $define PCF8574A

' Define the number of expanders connected. [1 to 8] (See ISIS schematic)
    $define PCF8574x_MaxNumber 8

' Define the interrupt PIC pin connected. (See ISIS schematic)
    $define _INT0_
'    $define _INT1_    
'    $define _INT2_    

'*******************************************************************************
'*******************************************************************************
' Library for I2C software commands.
' Define the I2C bus For AMICUS18 BOARD.
    $define _SDA_Pin PORTC.4
    $define _SCL_Pin PORTC.3

    '/ Declare the Speed or disable the line for slower speed.
    $define _Hbus_Bitrate_ 100

    '/ Declare the pin to enable/disable the encoder
    $define EncoderPinEnable PORTA.0

'/==============================================================================
    Include "PCF8574A-I2C-2.Inc"
'/==============================================================================
' Declare some variables
    Dim DemoFlags As Byte
    Dim INT_error_Flag   As DemoFlags.0
    Dim PCAInterruptPORT As DemoFlags.1

    Dim CheckAck As Byte
    Dim ReadPCA As Byte
    Dim NumberExpander As Byte    
    Dim PCFTemp As Byte

    '/ The aGPIO array get the value of inputs of the expanders. (If necessary)
    $if PCF8574x_MaxNumber = 1
    Dim aGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define aGPIO0 aGPIO[0]
    $endif

    $if PCF8574x_MaxNumber = 2
    Dim aGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define aGPIO0 aGPIO[0]
    $define aGPIO1 aGPIO[1]
    $endif

    $if PCF8574x_MaxNumber = 3
    Dim aGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define aGPIO0 aGPIO[0]
    $define aGPIO1 aGPIO[1]
    $define aGPIO2 aGPIO[2]
    $endif

    $if PCF8574x_MaxNumber = 4
    Dim aGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define aGPIO0 aGPIO[0]
    $define aGPIO1 aGPIO[1]
    $define aGPIO2 aGPIO[2]
    $define aGPIO3 aGPIO[3]
    $endif

    $if PCF8574x_MaxNumber = 5
    Dim aGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define aGPIO0 aGPIO[0]
    $define aGPIO1 aGPIO[1]
    $define aGPIO2 aGPIO[2]
    $define aGPIO3 aGPIO[3]
    $define aGPIO4 aGPIO[4]
    $endif

    $if PCF8574x_MaxNumber = 6
    Dim aGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define aGPIO0 aGPIO[0]
    $define aGPIO1 aGPIO[1]
    $define aGPIO2 aGPIO[2]
    $define aGPIO3 aGPIO[3]
    $define aGPIO4 aGPIO[4]
    $define aGPIO5 aGPIO[5]
    $endif

    $if PCF8574x_MaxNumber = 7
    Dim aGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define aGPIO0 aGPIO[0]
    $define aGPIO1 aGPIO[1]
    $define aGPIO2 aGPIO[2]
    $define aGPIO3 aGPIO[3]
    $define aGPIO4 aGPIO[4]
    $define aGPIO5 aGPIO[5]
    $define aGPIO6 aGPIO[6]
    $endif

    $if PCF8574x_MaxNumber = 8
    Dim aGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define aGPIO0 aGPIO[0]
    $define aGPIO1 aGPIO[1]
    $define aGPIO2 aGPIO[2]
    $define aGPIO3 aGPIO[3]
    $define aGPIO4 aGPIO[4]
    $define aGPIO5 aGPIO[5]
    $define aGPIO6 aGPIO[6]
    $define aGPIO7 aGPIO[7]
    $endif
    '/--------------------------------------------------------------------------
    '/ The aOLAT array get the value of outputs of the expanders. (If necessary)
    $if PCF8574x_MaxNumber = 1
    Dim aOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define aOLAT0 aOLAT[0]
    $endif

    $if PCF8574x_MaxNumber = 2
    Dim aOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define aOLAT0 aOLAT[0]
    $define aOLAT1 aOLAT[1]
    $endif

    $if PCF8574x_MaxNumber = 3
    Dim aOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define aOLAT0 aOLAT[0]
    $define aOLAT1 aOLAT[1]
    $define aOLAT2 aOLAT[2]
    $endif

    $if PCF8574x_MaxNumber = 4
    Dim aOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define aOLAT0 aOLAT[0]
    $define aOLAT1 aOLAT[1]
    $define aOLAT2 aOLAT[2]
    $define aOLAT3 aOLAT[3]
    $endif

    $if PCF8574x_MaxNumber = 5
    Dim aOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define aOLAT0 aOLAT[0]
    $define aOLAT1 aOLAT[1]
    $define aOLAT2 aOLAT[2]
    $define aOLAT3 aOLAT[3]
    $define aOLAT4 aOLAT[4]
    $endif

    $if PCF8574x_MaxNumber = 6
    Dim aOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define aOLAT0 aOLAT[0]
    $define aOLAT1 aOLAT[1]
    $define aOLAT2 aOLAT[2]
    $define aOLAT3 aOLAT[3]
    $define aOLAT4 aOLAT[4]
    $define aOLAT5 aOLAT[5]
    $endif

    $if PCF8574x_MaxNumber = 7
    Dim aOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define aOLAT0 aOLAT[0]
    $define aOLAT1 aOLAT[1]
    $define aOLAT2 aOLAT[2]
    $define aOLAT3 aOLAT[3]
    $define aOLAT4 aOLAT[4]
    $define aOLAT5 aOLAT[5]
    $define aOLAT6 aOLAT[6]
    $endif

    $if PCF8574x_MaxNumber = 8
    Dim aOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define aOLAT0 aOLAT[0]
    $define aOLAT1 aOLAT[1]
    $define aOLAT2 aOLAT[2]
    $define aOLAT3 aOLAT[3]
    $define aOLAT4 aOLAT[4]
    $define aOLAT5 aOLAT[5]
    $define aOLAT6 aOLAT[6]
    $define aOLAT7 aOLAT[7]
    $endif

'*******************************************************************************    
    On_Hardware_Interrupt GoTo PCA_INT_Handler
'*******************************************************************************  
' Configure the virtual pins of the PCF8574 expander.
' Input Pins: Name, Number of the Pin of a PORT for interrupt register
' Detect the Buttons by falling edge input of rising edge for one PORT [0 to 7].
'/ Expander 1:
    Declare_InputMode8(Button1,0) 
    Declare_InputMode8(Button2,1) 
    Declare_InputMode8(Button3,2) 
    Declare_InputMode8(Button4,3) 
'/ Expander 2:
    Declare_InputMode8(Button5,0) 
    Declare_InputMode8(Button6,1) 
    Declare_InputMode8(Button7,2) 
    Declare_InputMode8(Button8,3) 
'/ Expander 3:
    Declare_InputMode8(Button9,0) 
    Declare_InputMode8(Button10,1) 
    Declare_InputMode8(Button11,2) 
    Declare_InputMode8(Button12,3)
'/ Expander 4:
    Declare_InputMode8(Button13,0) 
    Declare_InputMode8(Button14,1) 
    Declare_InputMode8(Button15,2) 
    Declare_InputMode8(Button16,3)
'/ Expander 5:
    Declare_InputMode8(Button17,0) 
    Declare_InputMode8(Button18,1) 
    Declare_InputMode8(Button19,2) 
    Declare_InputMode8(Button20,3)
'/ Expander 6:
    Declare_InputMode8(Button21,0) 
    Declare_InputMode8(Button22,1) 
    Declare_InputMode8(Button23,2) 
    Declare_InputMode8(Button24,3)
'/ Expander 7:
    Declare_InputMode8(Button25,0) 
    Declare_InputMode8(Button26,1) 
    Declare_InputMode8(Button27,2) 
    Declare_InputMode8(Button28,3)
'/ Expander 8:
    Declare_InputMode8(Button29,0) 
    Declare_InputMode8(Button30,1) 
    Declare_InputMode8(Button31,2) 
    Declare_InputMode8(Button32,3)
        
' Output Pins: Name, Number of the Pin of one PORT [0 to 7]
'/ Expander 1:
    Declare_OutputExp0(LED1,4)
    Declare_OutputExp0(LED2,5)
    Declare_OutputExp0(LED3,6)
    Declare_OutputExp0(LED4,7)
    '/ Result => LED1#pin, LED2#pin, LED3#pin, LED4#pin
'/ Expander 2:
    Declare_OutputExp1(LED5,4)
    Declare_OutputExp1(LED6,5)
    Declare_OutputExp1(LED7,6)
    Declare_OutputExp1(LED8,7)
    '/ Result => LED4#pin, LED5#pin, LED6#pin, LED8#pin
'/ Expander 3:
    Declare_OutputExp2(LED9,4)
    Declare_OutputExp2(LED10,5)
    Declare_OutputExp2(LED11,6)
    Declare_OutputExp2(LED12,7)
    '/ Result => LED9#pin, LED10#pin, LED11#pin, LED12#pin
'/ Expander 4:
    Declare_OutputExp3(LED13,4)
    Declare_OutputExp3(LED14,5)
    Declare_OutputExp3(LED15,6)
    Declare_OutputExp3(LED16,7)
    '/ Result => LED13#pin, LED14#pin, LED15#pin, LED16#pin
'/ Expander 5:
    Declare_OutputExp4(LED17,4)
    Declare_OutputExp4(LED18,5)
    Declare_OutputExp4(LED19,6)
    Declare_OutputExp4(LED20,7)
    '/ Result => LED17#pin, LED18#pin, LED19#pin, LED20#pin
'/ Expander 6:
    Declare_OutputExp5(LED21,4)
    Declare_OutputExp5(LED22,5)
    Declare_OutputExp5(LED23,6)
    Declare_OutputExp5(LED24,7)
    '/ Result => LED21#pin, LED22#pin, LED23#pin, LED24#pin
'/ Expander 7:
    Declare_OutputExp6(LED25,4)
    Declare_OutputExp6(LED26,5)
    Declare_OutputExp6(LED27,6)
    Declare_OutputExp6(LED28,7)
    '/ Result => LED25#pin, LED26#pin, LED27#pin, LED28#pin
'/ Expander 8:
    Declare_OutputExp7(LED29,4)
    Declare_OutputExp7(LED30,5)
    Declare_OutputExp7(LED31,6)
    Declare_OutputExp7(LED32,7)
    '/ Result => LED25#pin, LED26#pin, LED27#pin, LED28#pin
         
'*******************************************************************************
'*******************************************************************************
INIT_PROJECT:
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    DelayMS 250
    ' Disable Interrupts
    INTCON = 0      
    WDTCON = 0      ' Disable the Watchdog
    ' Clear all PORTS
    PORTA = 0       
    LATA = 0
    PORTB = 0
    LATB = 0    
    PORTC = 0
    LATC = 0
    ' Declare all digital   
    ADCON0 = 0
    ADCON1 = 15
    ADCON2 = 0    
    CM1CON0 = 0
    CM2CON0 = 0
    CM2CON1 = 0    
    $ifdef _ANSEL 
    ANSEL = 0
    ANSELH = 0
    $endif
    $ifdef _ANSELA
    ANSELA = 0
    $endif
    $ifdef _ANSELB
    ANSELB = 0
    $endif
    $ifdef _ANSELC
    ANSELC = 0
    $endif
    $ifdef _ANSELD
    ANSELD = 0
    $endif    
    $ifdef _ANSELE
    ANSELE = 0
    $endif    
    ' Disable Pull-up resistors in PORTB
    INTCON2bits_NOT_RBPU = 1    

' Init the eeprom
    $ifdef _EECON1
    EECON1 = 0
    $endif
    $ifdef _EEADR
    EEADR = 0
    $endif
    $ifdef _EEDATA
    EEDATA = 0
    $endif
    $ifdef _EECON2
    EECON2 = 0
    $endif
    PIR1 = 0
    PIR2 = 0    
    
    ' Setup the Interrupts
    $ifdef _RCON
    RCON = 0            ' IPEN = 0, only High priority interrupts
    $endif
    INTCON3 = 0    
    Input PORTB
    DemoFlags = 0
'/============================================================================== 
' Configuration for beginner USERS. 
ConfigPCA:
    '/ Configure the Inputs/Outputs
    $if PCF8574x_MaxNumber > 0
    Config_IODIR0(P7_Output, P6_Output, P5_Output, P4_Output, P3_Input, P2_Input, P1_Input, P0_Input)
    $endif

    $if PCF8574x_MaxNumber > 1
    Config_IODIR1(P7_Output, P6_Output, P5_Output, P4_Output, P3_Input, P2_Input, P1_Input, P0_Input)
    $endif

    $if PCF8574x_MaxNumber > 2
    Config_IODIR2(P7_Output, P6_Output, P5_Output, P4_Output, P3_Input, P2_Input, P1_Input, P0_Input)
    $endif

    $if PCF8574x_MaxNumber > 3
    Config_IODIR3(P7_Output, P6_Output, P5_Output, P4_Output, P3_Input, P2_Input, P1_Input, P0_Input)
    $endif

    $if PCF8574x_MaxNumber > 4
    Config_IODIR4(P7_Output, P6_Output, P5_Output, P4_Output, P3_Input, P2_Input, P1_Input, P0_Input)    
    $endif

    $if PCF8574x_MaxNumber > 5
    Config_IODIR5(P7_Output, P6_Output, P5_Output, P4_Output, P3_Input, P2_Input, P1_Input, P0_Input)
    $endif

    $if PCF8574x_MaxNumber > 6
    Config_IODIR6(P7_Output, P6_Output, P5_Output, P4_Output, P3_Input, P2_Input, P1_Input, P0_Input)
    $endif

    $if PCF8574x_MaxNumber > 7
    Config_IODIR7(P7_Output, P6_Output, P5_Output, P4_Output, P3_Input, P2_Input, P1_Input, P0_Input)        
    $endif
    '/ Configure the Inputs Polarity
    '/ The inputs are inverted because all buttons are connected to 0V.
    '/ If Buttons are open to ground at normal state then write the configuration => PX_Open
    '/ If Buttons are closed to ground At normal state then write the configuration => PX_Closed
    '/ Else for outputs => PX_None
    $if PCF8574x_MaxNumber > 0
    Config_IPOL0(P7_None, P6_None, P5_None, P4_None, P3_Open, P2_Open, P1_Open, P0_Open)
    $endif

    $if PCF8574x_MaxNumber > 1
    Config_IPOL1(P7_None, P6_None, P5_None, P4_None, P3_Open, P2_Open, P1_Open, P0_Open)
    $endif

    $if PCF8574x_MaxNumber > 2
    Config_IPOL2(P7_None, P6_None, P5_None, P4_None, P3_Open, P2_Open, P1_Open, P0_Open)
    $endif

    $if PCF8574x_MaxNumber > 3
    Config_IPOL3(P7_None, P6_None, P5_None, P4_None, P3_Open, P2_Open, P1_Open, P0_Open)
    $endif

    $if PCF8574x_MaxNumber > 4
    Config_IPOL4(P7_None, P6_None, P5_None, P4_None, P3_Open, P2_Open, P1_Open, P0_Open)
    $endif

    $if PCF8574x_MaxNumber > 5
    Config_IPOL5(P7_None, P6_None, P5_None, P4_None, P3_Open, P2_Open, P1_Open, P0_Open)
    $endif

    $if PCF8574x_MaxNumber > 6
    Config_IPOL6(P7_None, P6_None, P5_None, P4_None, P3_Open, P2_Open, P1_Open, P0_Open)
    $endif

    $if PCF8574x_MaxNumber > 7
    Config_IPOL7(P7_None, P6_None, P5_None, P4_None, P3_Open, P2_Open, P1_Open, P0_Open)
    $endif
'///////////////////////////////////////////////////////////////////////////////
MAIN:
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    HRSOut "Testing the PCF8574-PCF8574A I2C IO Expander:",CR
    HRSOut "In Interrupt Mode 8 Bits with 1 INT pin (INT0/INT1/INT2).",CR,CR
    HRSOut "Code written by Alberto Freixanet.",CR
    HRSOut "Demo: Test_PCF8574A_INT-1.BAS 1.0 - 26/06/2023",CR
    HRSOut "Software I2C Library: ", _WhatLib, " ", _WhatVersion,CR
    HRSOut "Compiler Free Positron Version 4.0.1.8",CR
    HRSOut "Proteus VSM V8.2:",CR
    HRSOut "Tested with the PIC18F25K20",CR,CR

'*******************************************************************************
'*******************************************************************************
'/ Initialize the expander and check the transmision. (Mandatory)
    CheckAck = PCF8574_Init()

    '/ Control the ACK error of the I2C bus of the PCA9554.
    HRSOut "NACK = ",Hex2 CheckAck,CR
    If CheckAck = $0F Then
        HRSOut "NACK I2C comunication error!",CR
        ReadPCA = PCF8574_Read_NbExpander()
        HRSOut "Fault Expander Number: ", Dec ReadPCA,CR
        HRSOut "The PCF8574 is not initialized",CR
        HRSOut "The demo EXITS here",CR
        Stop
    ElseIf CheckAck = 0 Then    
        HRSOut "PCF8574 Received ACK OK: CONGRATULATIONS!!!",CR
    EndIf         
    CheckINTxPins()     '/ Show the state of the INTX pins. (not mandatory)
    HRSOut CR
 
'*******************************************************************************
'*******************************************************************************
'///////////////////////////////////////////////////////////////////////////////
' Testing the Expander Initialization:
'*******************************************************************************
'/ WARNING: DO NOT MODIFY THE CONFIGURATION REGISTERS DIRECTLY
'/ USE THE COMMANDS TO READ ALL THE REGISTERS OF THE EXPANDER
'*******************************************************************************
START_CODE:
    #ifdef WatchDog_Req
    Clrwdt
    #endif
'*******************************************************************************
'/ NOW TESTING ALL COMMANDS OF THE LIBRARY:
'*******************************************************************************
'    GoTo READEXPANDERS
' Testing the LEDS
    HRSOut "Lights all LEDS",CR

    '/ ReturnVar Outputs = Command(Data Out to write, Number Expander)
    $if PCF8574x_MaxNumber > 0
    ReadPCA = PCF8574_WritePORT($F0,0)          '/ Send Data + Number Expander 0.               
    HRSOut "Data EXP0: ", IBin8 ReadPCA, CR     
    DelayMS 1000
    $endif

    $if PCF8574x_MaxNumber > 1
    ReadPCA = PCF8574_WritePORT($F0,1)          '/ Send Data + Number Expander 1.               
    HRSOut "Data EXP1: ", IBin8 ReadPCA, CR     
    DelayMS 500
    $endif

    $if PCF8574x_MaxNumber > 2    
    ReadPCA = PCF8574_WritePORT($F0,2)          '/ Send Data + Number Expander 2.                
    HRSOut "Data EXP2: ", IBin8 ReadPCA, CR     
    DelayMS 500
    $endif        

    $if PCF8574x_MaxNumber > 3   
    ReadPCA = PCF8574_WritePORT($F0,3)          '/ Send Data + Number Expander 3.               
    HRSOut "Data EXP3: ", IBin8 ReadPCA, CR         
    DelayMS 500
    $endif           

    $if PCF8574x_MaxNumber > 4 
    ReadPCA = PCF8574_WritePORT($F0,4)          '/ Send Data + Number Expander 4.              
    HRSOut "Data EXP4: ", IBin8 ReadPCA, CR     
    DelayMS 500
    $endif    

    $if PCF8574x_MaxNumber > 5 
    ReadPCA = PCF8574_WritePORT($F0,5)          '/ Send Data + Number Expander 5.              
    HRSOut "Data EXP5: ", IBin8 ReadPCA, CR      
    DelayMS 500
    $endif         

    $if PCF8574x_MaxNumber > 6 
    ReadPCA = PCF8574_WritePORT($F0,6)          '/ Send Data + Number Expander 6.               
    HRSOut "Data EXP6: ", IBin8 ReadPCA, CR        
    DelayMS 500
    $endif     

    $if PCF8574x_MaxNumber > 7
    ReadPCA = PCF8574_WritePORT($F0,7)          '/ Send Data + Number Expander 7.               
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR       
    DelayMS 500
    $endif 
    
    HRSOut "Waiting 2 seconds...",CR
    DelayMS 2000

    HRSOut "Clear all LEDS",CR
    $if PCF8574x_MaxNumber > 0
    ReadPCA = PCF8574_WritePORT(0,0)            '/ Send Data + Number Expander 0.                  
    HRSOut "Data EXP0: ", IBin8 ReadPCA, CR    
    DelayMS 500
    $endif

    $if PCF8574x_MaxNumber > 1
    ReadPCA = PCF8574_WritePORT(0,1)            '/ Send Data + Number Expander 1.               
    HRSOut "Data EXP1: ", IBin8 ReadPCA, CR   
    DelayMS 500
    $endif

    $if PCF8574x_MaxNumber > 2    
    ReadPCA = PCF8574_WritePORT(0,2)            '/ Send Data + Number Expander 2.               
    HRSOut "Data EXP2: ", IBin8 ReadPCA, CR    
    DelayMS 500
    $endif
    
    $if PCF8574x_MaxNumber > 3             
    ReadPCA = PCF8574_WritePORT(0,3)            '/ Send Data + Number Expander 3.              
    HRSOut "Data EXP3: ", IBin8 ReadPCA, CR     
    DelayMS 500
    $endif

    $if PCF8574x_MaxNumber > 4        
    ReadPCA = PCF8574_WritePORT(0,4)            '/ Send Data + Number Expander 4.             
    HRSOut "Data EXP4: ", IBin8 ReadPCA, CR      
    DelayMS 500
    $endif    

    $if PCF8574x_MaxNumber > 5
    ReadPCA = PCF8574_WritePORT(0,5)            '/ Send Data + Number Expander 5.               
    HRSOut "Data EXP5: ", IBin8 ReadPCA, CR     
    DelayMS 500
    $endif        

    $if PCF8574x_MaxNumber > 6
    ReadPCA = PCF8574_WritePORT(0,6)            '/ Send Data + Number Expander 6.               
    HRSOut "Data EXP6: ", IBin8 ReadPCA, CR        
    DelayMS 500
    $endif    

    $if PCF8574x_MaxNumber > 7
    ReadPCA = PCF8574_WritePORT(0,7)            '/ Send Data + Number Expander 7.               
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR      
    DelayMS 1000
    $endif

    $if PCF8574x_MaxNumber > 7
    HRSOut "SET ALL LEDS EXP:32",CR
    ReadPCA = PCF8574_WritePORT($F0,7)          '/ Send Data + Number Expander 7.               
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR     
    DelayMS 1000
    $endif

    $if PCF8574x_MaxNumber > 7
    HRSOut "Clear LED32",CR
    ReadPCA = PCF8574_WritePinPORT(LED32#pin,0,7)   '/ Send Pin + State[0/1] + Number Expander 7. 
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR
    DelayMS 1000
    $endif

    $if PCF8574x_MaxNumber > 7
    HRSOut "Clear LED31",CR
    ReadPCA = PCF8574_WritePinPORT(LED31#pin,0,7)   '/ Send Pin + State[0/1] + Number Expander 7. 
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR     
    DelayMS 1000

    HRSOut "Clear LED30",CR
    ReadPCA = PCF8574_WritePinPORT(LED30#pin,0,7)   '/ Send Pin + State[0/1] + Number Expander 7.  
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR     
    DelayMS 1000
    
    HRSOut "Clear LED29",CR
    ReadPCA = PCF8574_WritePinPORT(LED29#pin,0,7)   '/ Send Pin + State[0/1] + Number Expander 7.  
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR     
    DelayMS 2000    

    HRSOut "Set LED29",CR
    ReadPCA = PCF8574_WritePinPORT(LED29#pin,1,7)   '/ Send Pin + State[0/1] + Number Expander 7.  
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR     
    DelayMS 1000 

    HRSOut "Set LED30",CR
    ReadPCA = PCF8574_WritePinPORT(LED30#pin,1,7)   '/ Send Pin + State[0/1] + Number Expander 7.  
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR     
    DelayMS 1000 

    HRSOut "Set LED31",CR
    ReadPCA = PCF8574_WritePinPORT(LED31#pin,1,7)   '/ Send Pin + State[0/1] + Number Expander 7.  
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR     
    DelayMS 1000 

    HRSOut "Set LED32",CR
    ReadPCA = PCF8574_WritePinPORT(LED32#pin,1,7)   '/ Send Pin + State[0/1] + Number Expander 7.  
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR     
    DelayMS 2000 
    $endif

    $if PCF8574x_MaxNumber > 7
    HRSOut "TOGGLE LEDS",CR
    HRSOut "Toggle LED32",CR
    ReadPCA = PCF8574_TogglePinPORT(LED32#pin,7)    '/ Send Pin + State[0/1] + Number Expander 7. 
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR     
    DelayMS 1000

    HRSOut "Toggle LED31",CR
    ReadPCA = PCF8574_TogglePinPORT(LED31#pin,7)    '/ Send Pin + State[0/1] + Number Expander 7. 
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR    
    DelayMS 1000

    HRSOut "Toggle LED30",CR
    ReadPCA = PCF8574_TogglePinPORT(LED30#pin,7)    '/ Send Pin + State[0/1] + Number Expander 7. 
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR     
    DelayMS 1000

    HRSOut "Toggle LED29",CR
    ReadPCA = PCF8574_TogglePinPORT(LED29#pin,7)    '/ Send Pin + State[0/1] + Number Expander 7. 
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR     
    DelayMS 1000

    HRSOut "LIGHT ALL LEDS EXPANDER: Expander 7",CR
    ReadPCA = PCF8574_WritePORT($F0,7)          '/ Send Data + Number Expander 0.               
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR     
    DelayMS 1000

    HRSOut "CLEAR ALL LEDS EXPANDER: Expander 7",CR
    ReadPCA = PCF8574_WritePORT($00,7)              '/ Send Data + Number Expander 7.                        
    HRSOut "Data EXP7: ", IBin8 ReadPCA, CR,CR   
    DelayMS 500
    $endif
'stop
'/==============================================================================
READEXPANDERS:
    '/ In Interrogation Mode the Return Var is the value of inputs to read.
    HRSOut "READ all Expanders",CR

    '/ ReturnVar Inputs = Command(Delay = 0, Number Expander)
    $if PCF8574x_MaxNumber > 0
    ReadPCA = PCF8574_ReadPORT(0,0)                   '/ Delay = 0 : Expander Nb = 0.
    HRSOut "Input Exp: 0: ", IBin8 ReadPCA,CR     
    HRSOut "IPOL0       : ", IBin8 vIPOL0,CR
    PCFTemp = PCF8574_Read_ControlWrite()              '/ Checking the control write
    HRSOut "Control Write: ", IBin8 PCFTemp,CR  
    PCFTemp = PCF8574_Read_ControlRead()               '/ Checking the control read
    HRSOut "Control Read:  ", IBin8 PCFTemp,CR,CR 
    $endif

    $if PCF8574x_MaxNumber > 1
    ReadPCA = PCF8574_ReadPORT(0,1)                   '/ Delay = 0 : Expander Nb = 1.
    HRSOut "Input Exp: 1: ", IBin8 ReadPCA,CR    
    HRSOut "IPOL1       : ", IBin8 vIPOL1,CR,CR
    $endif

    $if PCF8574x_MaxNumber > 2
    ReadPCA = PCF8574_ReadPORT(0,2)                   '/ Delay = 0 : Expander Nb = 2.
    HRSOut "Input Exp: 2: ", IBin8 ReadPCA,CR    
    HRSOut "IPOL2       : ", IBin8 vIPOL2,CR,CR
    $endif

    $if PCF8574x_MaxNumber > 3
    ReadPCA = PCF8574_ReadPORT(0,3)                   '/ Delay = 0 : Expander Nb = 3.
    HRSOut "Input Exp: 3: ", IBin8 ReadPCA,CR    
    HRSOut "IPOL3       : ", IBin8 vIPOL3,CR,CR
    $endif

    $if PCF8574x_MaxNumber > 4
    ReadPCA = PCF8574_ReadPORT(0,4)                   '/ Delay = 0 : Expander Nb = 4.
    HRSOut "Input Exp: 4: ", IBin8 ReadPCA,CR    
    HRSOut "IPOL4       : ", IBin8 vIPOL4,CR,CR
    $endif

    $if PCF8574x_MaxNumber > 5
    ReadPCA = PCF8574_ReadPORT(0,5)                   '/ Delay = 0 : Expander Nb = 5.
    HRSOut "Input Exp: 5: ", IBin8 ReadPCA,CR    
    HRSOut "IPOL5       : ", IBin8 vIPOL5,CR,CR
    $endif

    $if PCF8574x_MaxNumber > 6
    ReadPCA = PCF8574_ReadPORT(0,6)                   '/ Delay = 0 : Expander Nb = 6.
    HRSOut "Input Exp: 6: ", IBin8 ReadPCA,CR    
    HRSOut "IPOL6       : ", IBin8 vIPOL6,CR,CR
    $endif

    $if PCF8574x_MaxNumber > 7
    ReadPCA = PCF8574_ReadPORT(0,7)                   '/ Delay = 0 : Expander Nb = 7.
    HRSOut "Input Exp: 7: ", IBin8 ReadPCA,CR      
    HRSOut "IPOL7       : ", IBin8 vIPOL7,CR,CR
    $endif
    '---------------------------------------------------------------------------
    HRSOut "END checking the expander",CR
    HRSOut "Demo for Mode Interrupts & debounce Delay",CR
    HRSOut CR
'===============================================================================
'/ WAITING AN INTERRUPT:
'*******************************************************************************
'* WARNING: Write to an expander while the code is waiting an interrupt by a Button,
'* this makes an interrupt!
'*******************************************************************************
(*
'/ Interrupt output (INT)
    The PCF8574/74A provides an open-drain output (INT) which can be fed to a
    corresponding input of the microcontroller. As soon as a port input is
    changed, the INT will be active (LOW) and notify the microcontroller.

    An interrupt is generated at any rising or falling edge of the port inputs.
    After time tv(Q), the signal INT is valid.

    The interrupt will reset to HIGH when data on the port is changed to the original setting or
    data is read or written by the master.
    In the Write mode, the interrupt may be reset (HIGH) on the rising edge of the
    acknowledge bit of the data byte and also on the rising edge of the write to port pulse. The
    interrupt will always be reset (HIGH) on the falling edge of the write to port pulse (see
    Figure 8).

    The interrupt is reset (HIGH) in the Read mode on the rising edge of the acknowledge of
    slave address byte and on the rising edge of the read from port pulse.

    During the interrupt reset, any I/O change close to the read or write pulse may not
    generate an interrupt, or the interrupt will have a very short pulse. After the interrupt is
    reset, any change in I/Os will be detected and transmitted as an INT.

    At power-on reset all ports are in Input mode and the initial state of the ports is HIGH,
    therefore, for any port pin that is pulled LOW or driven LOW by external source, the
    interrupt output will be active (output LOW).
*)
'*******************************************************************************
'* WARNING:
'*In the Proteus VSM, write to PORT causes a false interrupt INT0 that it is overriden by the code.
'*******************************************************************************
START_MAIN:
    PCF8574_Clear_INTxIF()                     '/ Enable INT0/INT1/INT2 pin interrupt
    Do  '/ Do...Loop => Waiting Interrupts: Mandatory.
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        If INT_error_Flag = True Then           ' Check a false triggered interrupt.
            INT_error_Flag = False 
            HRSOut "Interrupt Error",CR,CR
        EndIf        
        '/----------------------------------------------------------------------
        '/ Read the expander in interrupt mode.
        If PCAInterruptPORT = True Then                 ' An interrupt occured?
            PCAInterruptPORT = False
            NumberExpander = Read_HC148(EncoderPORT)    '/ read the encoder HC148 to know where interrupt comes from.
 
            ' Then the inputs PORT will be read 35mS after interrupt. 
            '/ 35 mS for debounce.
            '/ NumberExpander variable is the number of Expander that has caused the interrupt.

            PCF8574_ReadPORT(35,NumberExpander) '/ Read the PORT (GPIO) with debounce.            
            '/ The result of inputs pins is into the vGPIO[NumberExpander] array.
            $ifdef _IntDebug
            If vPCA_ACK = $0F Then
                HRSOut "NACK error!",CR '/ Could be a comunication fault?
                Continue                '/ Skip all the codes below.
            EndIf
            If vPCA_ACK = $55 Then
                HRSOut "Number Expander error!",CR '/ This expander is not allowed!
                Continue                '/ Skip all the codes below.
            EndIf
            $endif

'            $ifdef _IntDebug
'            CheckINTxPins()         '/ Print the state of INTX pin of the PIC. (not mandatory)
'            $endif

            SkipAfterUserCode()     '/ (mandatory) In the second interrupt, the CheckInputsPORT() is disabled.
            '/==================================================================
            '/ Start User Code.
            CheckInputsPORT(NumberExpander)     '/ YES, real interrupt, then check the Buttons for this expander.
            '/ End User Code.
            '/==================================================================
        EndIf                                   
        '/----------------------------------------------------------------------        
        '/ Here more interrupts flags for the user.


 

        '/----------------------------------------------------------------------
    Loop
'///////////////////////////////////////////////////////////////////////////////
    End
'*******************************************************************************
'*******************************************************************************
' Hardware interrupt routine: 
PCA_INT_Handler:
    Context Save
'/------------------------------------------------------------------------------
    INT_error_Flag = True           ' Set an interrupt error. If no INTx flag then it is a False Interrupt
'/------------------------------------------------------------------------------
'/ The Pin INT0 is used for High Interrupt Service.
'/ If INT0 is used.
$ifdef _INT0_
    If INT0IF = True Then           '/ If INT0 interrupt flag true
        INT0IF = False              '/ Clear the flag for next time
        PCAInterruptPORT = True
        INT_error_Flag = False      '/ This is a true interrupt then Clear the interrupt error
    EndIf
$endif
'/------------------------------------------------------------------------------
'/ The INT1 pin could be used for Low Interrupt Service.
'/ For that write the priority bit to 0 (INTCON3Bits_INT1IP = 0)
'/
'/ Now the INT1 Pin has the high priority to work with High Interrupt service: (INTCON3Bits_INT1IP = 1)  
'/ If INT1 is used.
$ifdef _INT1_
    If INT1IF = True Then           '/ If INT1 interrupt flag true
        INT1IF = False              '/ Clear the flag for next time
        PCAInterruptPORT = True     '/ Set an external interrupt flag for main program.
        INT_error_Flag = False      '/ This is a true interrupt then Clear the interrupt error
    EndIf
$endif
'/------------------------------------------------------------------------------
'/ The INT2 pin could be used for Low Interrupt Service.
'/ For that write the priority bit to 0 (INTCON3Bits_INT2IP = 0)
'/
'/ Now the INT2 Pin has the high priority to work with High Interrupt service: (INTCON3Bits_INT2IP = 1)  
'/ If INT2 is used.
$ifdef _INT2_
    If INT2IF = True Then           '/ If INT2 interrupt flag true
        INT2IF = False              '/ Clear the flag for next time
        PCAInterruptPORT = True     '/ Set an external interrupt flag for main program.
        INT_error_Flag = False      '/ This is a true interrupt then Clear the interrupt error
    EndIf
$endif
'/------------------------------------------------------------------------------
'/ More interrupts for the user.



'/------------------------------------------------------------------------------
    Context Restore
' End of the Hardware interrupt routine
OverHighInterrupts:
'*******************************************************************************
'///////////////////////////////////////////////////////////////////////////////

Proc CheckInputsPORT(pNbExp As Byte)
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: CheckInputsPORT16 : START")
    '/--------------------------------------------------------------------------
    Dim ReadPCA As Byte     '/ Local variable.
    Dim DataOut As Bit      '/ Local variable.

    #ifndef SkipOverCode
    #error "The Procedure [SkipAfterUserCode()] must to be placed after PCF8574_ReadPORT() Procedure!"
    #endif
'/------------------------------------------------------------------------------
    ' The definitions of the Buttons are based on the INTF Aregister. (Virtual input PORTA)
    $if PCF8574x_MaxNumber > 0
    If pNbExp = 0 Then
        If Button1 = True Then                          ' Check if Button 1 is pressed. (Virtual Input PORTA)
            If LED1 = False Then                        ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED1#pin,DataOut,0)     ' Writing the new value to the PORTA.
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button1 pressed",CR
            PrintIntRegisters()                         '/ Read the registers to control them.
            HRSOut "OLAT0: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif

        ElseIf Button2 = True Then                          ' Check if Button 2 is pressed. (Virtual Input PORTA)
            If LED2 = False Then                        ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED2#pin,DataOut,0) 
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button2 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT0: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif

        ElseIf Button3 = True Then                          ' Check if Button 3 is pressed. (Virtual Input PORTA)
            If LED3 = False Then                        ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED3#pin,DataOut,0)      
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button3 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT0: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif

        ElseIf Button4 = True Then                          ' Check if Button 4 is pressed. (Virtual Input PORTA)
            If LED4 = False Then                        ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED4#pin,DataOut,0)     
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button4 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT0: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        EndIf
    EndIf
    $endif
    '/==========================================================================
    $if PCF8574x_MaxNumber > 1
    If pNbExp = 1 Then
        If Button5 = True Then                          ' Check if Button 1 is pressed. (Virtual Input PORTA)
            If LED5 = False Then                        ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED5#pin,DataOut,1)     ' Writing the new value to the PORTA.
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button5 pressed",CR
            PrintIntRegisters()                         '/ Read the registers to control them.
            HRSOut "OLAT1: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
       
        ElseIf Button6 = True Then                          ' Check if Button 2 is pressed. (Virtual Input PORTA)
            If LED6 = False Then                        ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED6#pin,DataOut,1)     
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button6 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT1: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        
        ElseIf Button7 = True Then                          ' Check if Button 3 is pressed. (Virtual Input PORTA)
            If LED7 = False Then                        ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                             ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED7#pin,DataOut,1)   
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button7 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT1: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
    
        ElseIf Button8 = True Then                          ' Check if Button 4 is pressed. (Virtual Input PORTA)
            If LED8 = False Then                        ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED8#pin,DataOut,1)     
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button8 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT1: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        EndIf
    EndIf
    $endif
    '/==========================================================================
    $if PCF8574x_MaxNumber > 2
    If pNbExp = 2 Then
        If Button9 = True Then                          ' Check if Button 1 is pressed. (Virtual Input PORTA)
            If LED9 = False Then                        ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED9#pin,DataOut,2)     ' Writing the new value to the PORTA.
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button9 pressed",CR
            PrintIntRegisters()                         '/ Read the registers to control them.
            HRSOut "OLAT2: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
       
        ElseIf Button10 = True Then                         ' Check if Button 2 is pressed. (Virtual Input PORTA)
            If LED10 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED10#pin,DataOut,2)    
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button10 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT2: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        
        ElseIf Button11 = True Then                         ' Check if Button 3 is pressed. (Virtual Input PORTA)
            If LED11 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED11#pin,DataOut,2)     
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button11 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT2: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
    
        ElseIf Button12 = True Then                         ' Check if Button 4 is pressed. (Virtual Input PORTA)
            If LED12 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED12#pin,DataOut,2)     
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button12 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT2: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        EndIf
    EndIf
    $endif
    '/==========================================================================
    $if PCF8574x_MaxNumber > 3
    If pNbExp = 3 Then
        If Button13 = True Then                         ' Check if Button 1 is pressed. (Virtual Input PORTA)
            If LED13 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED13#pin,DataOut,3)     ' Writing the new value to the PORTA.
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button13 pressed",CR
            PrintIntRegisters()                         '/ Read the registers to control them.
            HRSOut "OLAT3: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
       
        ElseIf Button14 = True Then                         ' Check if Button 2 is pressed. (Virtual Input PORTA)
            If LED14 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED14#pin,DataOut,3)      
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button14 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT3: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        
        ElseIf Button15 = True Then                         ' Check if Button 3 is pressed. (Virtual Input PORTA)
            If LED15 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED15#pin,DataOut,3)    
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button15 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT3: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
    
        ElseIf Button16 = True Then                         ' Check if Button 4 is pressed. (Virtual Input PORTA)
            If LED16 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED16#pin,DataOut,3)      
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button16 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT3: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        EndIf
    EndIf
    $endif
    '/==========================================================================
    $if PCF8574x_MaxNumber > 4
    If pNbExp = 4 Then
        If Button17 = True Then                         ' Check if Button 1 is pressed. (Virtual Input PORTA)
            If LED17 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED17#pin,DataOut,4)     ' Writing the new value to the PORTA.
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button17 pressed",CR
            PrintIntRegisters()                         '/ Read the registers to control them.
            HRSOut "OLAT4: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
       
        ElseIf Button18 = True Then                         ' Check if Button 2 is pressed. (Virtual Input PORTA)
            If LED18 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED18#pin,DataOut,4)      
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button18 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT4: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        
        ElseIf Button19 = True Then                         ' Check if Button 3 is pressed. (Virtual Input PORTA)
            If LED19 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED19#pin,DataOut,4)     
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button19 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT4: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
    
        ElseIf Button20 = True Then                         ' Check if Button 4 is pressed. (Virtual Input PORTA)
            If LED20 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED20#pin,DataOut,4)      
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button20 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT4: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        EndIf
    EndIf
    $endif    
    '/==========================================================================
    $if PCF8574x_MaxNumber > 5
    If pNbExp = 5 Then
        If Button21 = True Then                         ' Check if Button 1 is pressed. (Virtual Input PORTA)
            If LED21 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED21#pin,DataOut,5)     ' Writing the new value to the PORT.
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button21 pressed",CR
            PrintIntRegisters()                         '/ Read the registers to control them.
            HRSOut "OLAT5: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
       
        ElseIf Button22 = True Then                         ' Check if Button 2 is pressed. (Virtual Input PORTA)
            If LED22 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED22#pin,DataOut,5)     
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button22 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT5: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        
        ElseIf Button23 = True Then                         ' Check if Button 3 is pressed. (Virtual Input PORTA)
            If LED23 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED23#pin,DataOut,5)     
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button23 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT5: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
    
        ElseIf Button24 = True Then                         ' Check if Button 4 is pressed. (Virtual Input PORTA)
            If LED24 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED24#pin,DataOut,5)      
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button24 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT5: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        EndIf
    EndIf
    $endif
    '/==========================================================================
    $if PCF8574x_MaxNumber > 6
    If pNbExp = 6 Then
        If Button25 = True Then                         ' Check if Button 1 is pressed. (Virtual Input PORTA)
            If LED25 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED25#pin,DataOut,6)     ' Writing the new value to the PORTA.
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button25 pressed",CR
            PrintIntRegisters()                         '/ Read the registers to control them.
            HRSOut "OLAT6: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
       
        ElseIf Button26 = True Then                         ' Check if Button 2 is pressed. (Virtual Input PORTA)
            If LED26 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED26#pin,DataOut,6)     
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button26 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT6: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        
        ElseIf Button27 = True Then                         ' Check if Button 3 is pressed. (Virtual Input PORTA)
            If LED27 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED27#pin,DataOut,6)      
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button27 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT6: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
    
        ElseIf Button28 = True Then                         ' Check if Button 4 is pressed. (Virtual Input PORTA)
            If LED28 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED28#pin,DataOut,6)     
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button28 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT6: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        EndIf
    EndIf
    $endif
    '/==========================================================================
    $if PCF8574x_MaxNumber > 7
    If pNbExp = 7 Then
        If Button29 = True Then                         ' Check if Button 1 is pressed. (Virtual Input PORTA)
            If LED29 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED29#pin,DataOut,7)     ' Writing the new value to the PORTA.
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button29 pressed",CR
            PrintIntRegisters()                         '/ Read the registers to control them.
            HRSOut "OLAT7: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
       
        ElseIf Button30 = True Then                         ' Check if Button 2 is pressed. (Virtual Input PORTA)
            If LED30 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED30#pin,DataOut,7)     
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button30 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT7: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        
        ElseIf Button31 = True Then                         ' Check if Button 3 is pressed. (Virtual Input PORTA)
            If LED31 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED31#pin,DataOut,7)      
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button31 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT7: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
    
        ElseIf Button32 = True Then                         ' Check if Button 4 is pressed. (Virtual Input PORTA)
            If LED32 = False Then                       ' Change the state of the led.
                DataOut = True                          ' Set the LED 
            Else
                DataOut = False                         ' Clear the LED
            EndIf
            ReadPCA = PCF8574_WritePinPORT(LED32#pin,DataOut,7)    
            $ifdef _IntDebug
            HRSOut CR
            HRSOut "Button32 pressed",CR
            PrintIntRegisters()
            HRSOut "OLAT7: ", IBin8 ReadPCA,CR          ' See the bits in the terminal.
            $endif
        EndIf
    EndIf
    $endif
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: CheckInputsPORT16 : END")
    '/--------------------------------------------------------------------------
EndProc
'/==============================================================================
Proc PrintIntRegisters()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: PrintIntRegisters : START")
    '/--------------------------------------------------------------------------
    Dim ReadPCA As Byte     '/ Local Variable.

    ReadPCA = PCF8574_Read_GPIO_Shadow(NumberExpander)
    HRSOut "GPIO : ", IBin8 ReadPCA,CR 

    ReadPCA = PCF8574_Read_INTF_Shadow()
    HRSOut "xINTF: ",IBin8 ReadPCA,CR  

'    ReadPCA = PCF8574_Read_IODIR_Shadow(NumberExpander)
'    HRSOut "IODIR: ", IBin8 ReadPCA,CR  

'    ReadPCA = PCF8574_Read_IPOL_Shadow(NumberExpander)
'    HRSOut "IPOL : ", IBin8 ReadPCA,CR  
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: PrintIntRegisters : END")
    '/--------------------------------------------------------------------------
EndProc
'///////////////////////////////////////////////////////////////////////////////
    


